/**
 * StreitgenossenXJustizLesenAction.java
 * eu.gronos.kostenrechner.view.panel (Kostenrechner)
 */
package eu.gronos.kostenrechner.controller.baumbach;

import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.io.File;

import javax.swing.JFileChooser;
import javax.swing.JFrame;

import eu.gronos.kostenrechner.controller.BeschriebeneAktion;
import eu.gronos.kostenrechner.controller.files.LadeXjustizController;
import eu.gronos.kostenrechner.interfaces.StreitgenossenTabellenBehandelnd;
import eu.gronos.kostenrechner.model.LangBeschriftung;
import eu.gronos.kostenrechner.model.tenordaten.VerfahrensDatenContainer;
import eu.gronos.kostenrechner.view.baumbach.BaumbachAssistentDialog;
import eu.gronos.kostenrechner.view.baumbach.StreitgenossenPanel;
import eu.gronos.kostenrechner.view.baumbach.XjustizFileChooser;

/**
 * Der AbstractAction importiert eine XJustiz-Datei und ruft dann den
 * Baumbach-Assistenten auf. {@link KeyEvent#VK_X}
 *
 * @author Peter Schuster (setrok)
 * @date 10.11.2018
 *
 */
public class StreitgenossenXJustizLesenAction extends BeschriebeneAktion {
	private final StreitgenossenTabellenBehandelnd handler;
	private final JFrame frame;
	private static final long serialVersionUID = 3372100502718641247L;

	/**
	 * Der Konstruktor setzt die Felder der {@link BeschriebeneAktion}
	 * 
	 * @param handler      ein Bezug auf das aufrufende {@link StreitgenossenPanel}
	 * @param frame        TODO
	 * @param beschriftung eine {@link LangBeschriftung} mit
	 *                     ({@link javax.swing.Action#MNEMONIC_KEY}) und
	 *                     ({@link javax.swing.Action#ACCELERATOR_KEY})
	 */
	public StreitgenossenXJustizLesenAction(StreitgenossenTabellenBehandelnd handler, JFrame frame,
			LangBeschriftung beschriftung) {
		super(beschriftung);
		this.handler = handler;
		this.frame = frame;
	}

	@Override
	public void actionPerformed(ActionEvent ae) {
		File auswahl = null;
		VerfahrensDatenContainer container = null;
		BaumbachAssistentDialog bad = null;
		
		//JFileChooser jfc = new JFileChooser();
		//jfc.setFileFilter(new FileNameExtensionFilter("XML", "xml"));
		
		JFileChooser jfc = new XjustizFileChooser();		
		int ergebnis = jfc.showOpenDialog(null);
		
		if (ergebnis == JFileChooser.APPROVE_OPTION) {
			auswahl = jfc.getSelectedFile();
			container = new LadeXjustizController(handler, this.frame).xJustizLesen(auswahl);
		} else {
			bad = new BaumbachAssistentDialog(this.frame, this.handler.isWiderKlageAnzeigen());
			container = bad.showDialog();
		}
		if (container != null && container.streitgenossen != null) {
			this.handler.setzeWerte(container.streitgenossen);
			handler.requestFocusInWindow();
		}
	}
}
// , mnemonic, acceleratorKey, int mnemonic, KeyStroke acceleratorKey
// String name,String shortDescription, String actionCommandKey,
// name, shortDescription, mnemonic, actionCommandKey, acceleratorKey);
// , KeyEvent.VK_X, KeyStroke.getKeyStroke(KeyEvent.VK_O, Kostenrechner.ctrlMask
// + ActionEvent.SHIFT_MASK)