/*
 * WiderklageCheckboxTextfieldHorcher.java
 * eu.gronos.kostenrechner.view.baumbach (Kostenrechner)
 */
package eu.gronos.kostenrechner.controller.baumbach;

import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFormattedTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import eu.gronos.kostenrechner.model.VorsorgendeBeschriftung;
import eu.gronos.kostenrechner.model.tenordaten.Beteiligter;

/**
 * Klasse dient dazu, adjustWiderklageCheckBox aufzurufen, wenn ein anderes
 * Geschlecht für den Beteiligten ausgewählt wird. So kann der Text der Checkbox
 * an den Genus der Auswahl angepasst werden.
 *
 * Guckt bei Klägern, ob sie an der Widwrklage beteiligt sind und gibt
 * dementsprechend die Textfelder frei.
 *
 * @author Peter Schuster (setrok)
 * @date 11 Apr 2019
 *
 */
public class WiderklageCheckboxTextfieldHorcher implements ChangeListener, ItemListener {
	private final JCheckBox cbAnWiderklageBeteiligt;
	private final JComboBox<Beteiligter> cmbTypGenusNumerus;
	private final JFormattedTextField ftfStreitwert;
	private final JFormattedTextField ftfUnterliegen;
	private final VorsorgendeBeschriftung beschriftung;

	/**
	 * Konstruktor
	 */
	public WiderklageCheckboxTextfieldHorcher(JComboBox<Beteiligter> cmbTypGenusNumerus,
			JCheckBox cbAnWiderklageBeteiligt, JFormattedTextField ftfStreitwert, JFormattedTextField ftfUnterliegen,
			VorsorgendeBeschriftung beschriftung) {
		this.cmbTypGenusNumerus = cmbTypGenusNumerus;
		this.cbAnWiderklageBeteiligt = cbAnWiderklageBeteiligt;
		this.ftfStreitwert = ftfStreitwert;
		this.ftfUnterliegen = ftfUnterliegen;
		this.beschriftung = beschriftung;
	}

	@Override
	public void stateChanged(ChangeEvent e) {
		adjustTextfieldEditability();
	}

	/**
	 * Die Methode itemStateChanged dient dazu, adjustWiderklageCheckBox aufzurufen,
	 * wenn ein anderes Geschlecht für den Beteiligten ausgewählt wird. So kann der
	 * Text der Checkbox an den Genus der Auswahl angepasst werden.
	 * 
	 * The listener interface for receiving item events. The class that is
	 * interested in processing an item event implements this interface. The object
	 * created with that class is then registered with a component using the
	 * component's addItemListener method. When an item-selection event occurs, the
	 * listener object's itemStateChanged method is invoked.
	 * 
	 * @param ie
	 * 
	 * @see java.awt.event.ItemListener#itemStateChanged(java.awt.event.ItemEvent)
	 */
	@Override
	public void itemStateChanged(ItemEvent ie) {
		// TODO: angeblich sind ActionListener besser für JComboBox
		adjustWiderklageCheckBox();
	}

	/**
	 * Die Methode adjustWiderklageCheckBox dient dazu, den Text der JCheckBox
	 * chckbxAnWiderklageBeteiligt an den BeteiligtenTyp anzupassen ("und
	 * Widerkläger/Widerbeklagter")
	 */
	private void adjustWiderklageCheckBox() {
		int genusNumerus = 0;// männlich = 0; weiblich = 1
		int typ = -1;
		if (cmbTypGenusNumerus != null) {
			final Beteiligter beteiligter = cmbTypGenusNumerus.getModel()
					.getElementAt(cmbTypGenusNumerus.getSelectedIndex());
			genusNumerus = beteiligter.getGenusNumerus();
			typ = beteiligter.getTyp();
		}
		switch (typ) {
		case Beteiligter.DRITTWIDERBEKLAGTE:// 1
			cbAnWiderklageBeteiligt.setSelected(true);
			cbAnWiderklageBeteiligt.setEnabled(false);
			// jetzt kommt ein fall-through
		case Beteiligter.KLAEGER:// 0
			beschriftung.changeTo(1 + genusNumerus);
			beschriftung.applyTo(cbAnWiderklageBeteiligt);
			break;
		case Beteiligter.BEKLAGTE:// 2
			beschriftung.changeTo(3 + genusNumerus);
			beschriftung.applyTo(cbAnWiderklageBeteiligt);
			break;
		default:
			beschriftung.changeTo(0);
			beschriftung.applyTo(cbAnWiderklageBeteiligt);
		}
	}
	// cbAnWiderklageBeteiligt.setText(String.format(BaumbachBeteiligter.UND_WK_WBK,
	// BaumbachBeteiligter.WIDERBEKLAGTEN_BEUGUNG[genusNumerus]));
	// cbAnWiderklageBeteiligt.setText(String.format(BaumbachBeteiligter.UND_WK_WBK,
	// BaumbachBeteiligter.WIDERKLAEGER_BEUGUNG[genusNumerus]));
	// cbAnWiderklageBeteiligt.setText("und Widerkläger/Widerbeklagter");

	/**
	 * Die Methode sperrt die Eingabe-Textfelder bei Klägern, sofern die Widerklage
	 * nicht aktiviert ist
	 */
	private void adjustTextfieldEditability() {
		if (cbAnWiderklageBeteiligt == null || !cbAnWiderklageBeteiligt.isVisible() || ftfStreitwert == null
				|| ftfUnterliegen == null) {
			return;
		}
		int typ = -1;
		if (cmbTypGenusNumerus != null) {
			final Beteiligter beteiligter = cmbTypGenusNumerus.getModel()
					.getElementAt(cmbTypGenusNumerus.getSelectedIndex());
			typ = beteiligter.getTyp();
		}
		if (typ == Beteiligter.KLAEGER) {
			if (cbAnWiderklageBeteiligt.isSelected()) {
				ftfStreitwert.setEnabled(true);
				ftfUnterliegen.setEnabled(true);
			} else {
				ftfStreitwert.setEnabled(false);
				ftfStreitwert.setText("0");
				ftfUnterliegen.setEnabled(false);
				ftfUnterliegen.setText("0");
			}
		}
	}
}