/**
 * GebuehrenAufstellungAction.java
 * eu.gronos.kostenrechner.view.panel (Kostenrechner)
 */
package eu.gronos.kostenrechner.controller.gebuehren;

import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

import javax.swing.Action;

import eu.gronos.kostenrechner.Kostenrechner;
import eu.gronos.kostenrechner.controller.BeschriebeneAktion;
import eu.gronos.kostenrechner.controller.files.VerfahrensDatenContainer2GUI;
import eu.gronos.kostenrechner.interfaces.GebuehrenTableTextfieldHandler;
import eu.gronos.kostenrechner.logic.gebuehren.GebuehrenBerechnungsAufstellung;
import eu.gronos.kostenrechner.model.LangBeschriftung;
import eu.gronos.kostenrechner.model.tenordaten.TenorDatenContainer;
import eu.gronos.kostenrechner.model.tenordaten.VerfahrensDatenContainer;
import eu.gronos.kostenrechner.view.gebuehren.GebuehrenBerechnungPanel;
import eu.gronos.kostenrechner.view.helpanderror.FehlerDialog;
import eu.gronos.kostenrechner.view.result.TenorDialog;

/**
 * Die {@link Action} erstellt eine Aufstellung der Gebühren und ihrer Summe zu
 * dem ausgewählten Streitwert. {@link KeyEvent#VK_E}
 *
 * @author Peter Schuster (setrok)
 * @date 04.11.2018
 *
 */
public class GebuehrenAufstellungAction extends BeschriebeneAktion {
	private final GebuehrenTableTextfieldHandler handler;
	private static final long serialVersionUID = 4793754967839528087L;

	/**
	 * Konstruktor.
	 * 
	 * @param handler      Verweis auf {@link GebuehrenBerechnungPanel}
	 * @param beschriftung eine {@link LangBeschriftung}, der auch
	 *                     ({@link javax.swing.Action#NAME}),
	 *                     ({@link javax.swing.Action#SHORT_DESCRIPTION}),
	 *                     {@link Action#MNEMONIC_KEY},
	 *                     ({@link javax.swing.Action#ACTION_COMMAND_KEY}) entnommen
	 *                     werden.
	 */
	public GebuehrenAufstellungAction(GebuehrenTableTextfieldHandler handler, LangBeschriftung beschriftung) {
		super(beschriftung);
		this.handler = handler;
	}

	@Override
	public void actionPerformed(ActionEvent ae) {
		handler.requestFocusInWindow();
		if (handler.getFormattedTextField(GebuehrenBerechnungPanel.FTF_STREITWERT) != null
				&& handler.getTableModel(GebuehrenBerechnungPanel.TABLE_GEBUEHREN) != null) {
			try {
				final VerfahrensDatenContainer container = new VerfahrensDatenContainer2GUI()
						.leseAlleWertefuerContainer();
				final TenorDatenContainer tenorDaten = new GebuehrenBerechnungsAufstellung(container).erzeugeContainer();
				new TenorDialog(Kostenrechner.getInstance(),
						tenorDaten).showDialog();
			} catch (NumberFormatException e) {
				// TODO Überflüssig? Werte prüfen! Werden woanders geprüft?
				FehlerDialog.zeigeFehler(e.getLocalizedMessage(), e);
			} catch (NullPointerException | IllegalArgumentException e) {
				FehlerDialog.zeigeFehler(e.getLocalizedMessage(), e);
			}
		}
	}
}
/*
 * Konstruktor: (kommentieren)
 * 
 * @param name {@link Action#NAME} @param shortDescription {@link
 * Action#SHORT_DESCRIPTION}@param actionCommandKey {@link
 * Action#ACTION_COMMAND_KEY} @param gebuehrenBerechnungPanel @param vkE public
 * GebuehrenAufstellungAction(GebuehrenTableTextfieldHandler handler, String
 * name, String shortDescription, int mnemonic, String actionCommandKey) {
 * super(name, shortDescription, mnemonic, actionCommandKey); this.handler =
 * handler; }
 */
// gttmRechner
// .gebuehrenBerechnung, container
// , int mnemonic, mnemonic