/**
 * SchreibeEmailXml.java
 * eu.gronos.kostenrechner.view (Kostenrechner)
 */
package eu.gronos.kostenrechner.controller.system;

import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

import javax.swing.Action;

import eu.gronos.kostenrechner.Kostenrechner;
import eu.gronos.kostenrechner.controller.files.BerechnungXmlDatei;
import eu.gronos.kostenrechner.controller.files.VerfahrensDatenContainer2GUI;
import eu.gronos.kostenrechner.model.LangBeschriftung;
import eu.gronos.kostenrechner.model.tenordaten.VerfahrensDatenContainer;

/**
 * {@link Action} lässt eine Beschwerde-E-Mail im Standard-E-Mail-Client über
 * {@link TenorUtils.emailSchreiben(String, String)} schreiben. Dabei werden
 * XML-Daten mitgeschickt. {@link KeyEvent.VK_T}
 *
 * @author Peter Schuster (setrok)
 * @date 11 Nov 2018
 *
 */
public class SchreibeEmailXml extends SchreibeEmail {
	private static final long serialVersionUID = -4300105137631815898L;

	/**
	 * Konstruktor.
	 * 
	 * 
	 * @param beschriftung eine {@link LangBeschriftung} mit
	 *                     {@link Action#NAME},{@link Action#SHORT_DESCRIPTION},{@link Action#ACTION_COMMAND_KEY}
	 *                     und {@link Action#MNEMONIC_KEY}
	 * 
	 */
	public SchreibeEmailXml(LangBeschriftung beschriftung) {
		super(beschriftung);
	}
	// * @param name shortDescription @param mnemonic @param actionCommandKey
	// , mnemonic, int mnemonic
	// String name, String shortDescription, String actionCommandKey,
	// super(name, shortDescription, mnemonic, actionCommandKey);

	@Override
	public void actionPerformed(ActionEvent e) {
		final VerfahrensDatenContainer container = new VerfahrensDatenContainer2GUI().leseAlleWertefuerContainer();
		final String subject = "Fehlverhalten in kostentenor.de " + Kostenrechner.KOSTENRECHNER_VERSION;
		// Wäre schön, wenn noch XML-Daten mitgeschickt würden.
		final String xmlString = new BerechnungXmlDatei().toXmlString(container);
		if (new DatenschutzNoergler(Kostenrechner.getInstance(), DatenschutzNoergler.OPTION_DSGVO_EMAIL_BASIS)
				.noergle()) {
			// Wenn dem Anhang nicht zugestimmt wird, einfache E-Mail schreiben.
			final StringBuilder body = new StringBuilder(
					"Wenn mit den eingegebenen Daten ein Fehler aufgetreten sein sollte, beschreiben Sie das Fehlverhalten des Programms bitte!\n\n");
			if (new DatenschutzNoergler(Kostenrechner.getInstance(), DatenschutzNoergler.OPTION_DSGVO_EMAIL_ANHANG)
					.noergle()) {
				// Bei Zustimmung auch XML mitschicken
				body.append(xmlString);
			}
			super.schreibe(subject, body.toString());
		}
	}
}