/*
 * NameMap.java
 * eu.gronos.kostenrechner.model (Kostenrechner)
 */
package eu.gronos.kostenrechner.model;

import java.io.InputStream;
import java.io.StringWriter;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import javax.xml.bind.JAXB;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlElements;
import javax.xml.bind.annotation.XmlRootElement;

/**
 * Beschriftungen in HashMap oä organisieren, damit man alles über die id
 * abrufen kann.
 */
@XmlRootElement(name = "beschriftungsSammlung")
@XmlAccessorType(XmlAccessType.PROPERTY)
public class NameContainerMap extends HashMap<Integer, NameContainer> {
	private static final long serialVersionUID = 8156338816766505017L;
	private Set<NameContainer> beSchreibSet = new HashSet<NameContainer>();
	public static long start;
	public static long end;

	public NameContainerMap() {
		super();
	}

	/**
	 * Konstruktor mit ressource
	 * 
	 * @param dateiname "resources/beschriftungen.xml"
	 */
	public NameContainerMap(String dateiname) {
		super();
		start = System.currentTimeMillis();
		InputStream stream = getClass().getClassLoader().getResourceAsStream(dateiname);
		convertToMap(this, readFromStream(stream).getBeSchreibSet());
		end = System.currentTimeMillis();	
	}

	/**
	 * @return gibt {@link #beSchreibSet} als {@link Set<NameContainer>} zurück.
	 */
	@XmlElements({ @XmlElement(name = "nurname", type = NameContainer.class),
			@XmlElement(name = "beschriftung", type = Beschriftung.class),
			@XmlElement(name = "langVorsorgend", type = LangVorsorgeBeschriftung.class),
			@XmlElement(name = "langBeschriftung", type = LangBeschriftung.class),
			@XmlElement(name = "vorsorgend", type = VorsorgendeBeschriftung.class),
			@XmlElement(name = "ohneTitel", type = UnBeschriftung.class) })
	@XmlElementWrapper(name = "beschriftungen")
	public Set<NameContainer> getBeSchreibSet() {
		return beSchreibSet;
	}

	/**
	 * @param beSchreibSet d. {@link #beSchreibSet}, d. gesetzt werden soll als
	 *                     {@link List<NameContainer>}.
	 */
	public void setBeSchreibSet(Set<NameContainer> beSchreibSet) {
		this.beSchreibSet = beSchreibSet;
	}

	/**
	 * Die Methode dient dazu, das {@link NameContainerMap#getBeSchreibSet()} in
	 * eine Map zu wandeln
	 * 
	 * @param map          eine (leere) {@link NameContainerMap}, aus der sich die
	 *                     Methode auch das
	 *                     {@link NameContainerMap#getBeSchreibSet()} zieht.
	 * @param beSchreibSet {@link #getBeSchreibSet()}
	 * @return dieselbe {@link NameContainerMap}, allerdings mit Werten.
	 */
	private Map<Integer, NameContainer> convertToMap(NameContainerMap map, Set<NameContainer> beSchreibSet) {
		for (NameContainer b : beSchreibSet) {
			map.put(b.getKey(), b);// Integer.valueOf(b.idIntValue())
		}
		// Kostenrechner.getLogger().info
		//System.out.println((String.format("Einlesen von %d dauerte %d ms.", (end - start))));	
		return map;
	}

	/**
	 * Hier wird die XML-Datei gelesen. ressource einlesen
	 * 
	 * @param stream ein {@link InputStream}
	 * @return eine {@link HashMap}
	 */
	private NameContainerMap readFromStream(InputStream stream) {
		if (stream == null)
			throw new NullPointerException("Der InputStream für die angegebene Resource darf nicht null sein.");
		NameContainerMap map = JAXB.unmarshal(stream, NameContainerMap.class);
		return map;
	}

	public String toXmlString() {
		StringWriter sw = new StringWriter();
		try {
			Marshaller marshaller = JAXBContext.newInstance(NameContainerMap.class).createMarshaller();
			marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, true);
			marshaller.marshal(this, sw);
		} catch (JAXBException e) {
			e.printStackTrace();
			return "Programmzustände konnten nicht in XML konvertiert werden: " + e.getMessage();
		}
		return sw.toString();
	}
}
//for (Component comp : menu.getComponents()) {
//if (comp != null) {
//	NameContainer cont = buildFrom(comp, zaehler);
//	if (cont != null)
//		set.add(cont);
//}
//zaehler++;
//}

//public static Set<NameContainer> buildFromM(JMenu menu, int zaehler) {
//Set<NameContainer> set = new TreeSet<NameContainer>();
//int i = 0;
//try {
//while (true) {
//	JMenuItem comp = menu.getItem(i);
//	if (comp != null) {
//		NameContainer cont = buildFrom(comp, zaehler);
//		if (cont != null)
//			set.add(cont);
//	}
//	zaehler++;
//	i++;
//}
//} catch (Exception e) {
//e.printStackTrace();
//}
//return set;
//}

//		if (dialog instanceof HinzufuegenDialog<?>) {
//			set.addAll(buildFrom(zaehler, ((HinzufuegenDialog<?>) dialog).getButtonPane().getComponents()));
//		}
//		zaehler += set.size();
//		set.addAll(buildFrom(zaehler, dialog.getRootPane().getComponents()));
//		zaehler += set.size();
//		final ActionMap actionMap = dialog.getRootPane().getActionMap();
//		for (Object key : actionMap.allKeys()) {
//			final Action action = actionMap.get(key);
//			if (action != null) {
//				final LangBeschriftung beschriftung = buildFrom(action);
//				if (beschriftung.getId() == null)
//					beschriftung.setId(Integer.valueOf(zaehler).toString());
//				set.add(beschriftung);
//				// beschriftung.setId(Integer.valueOf(beschriftung.getId().hashCode()).toString());
//			}
//		}
// new ArrayList<JComponent>();
// for (Component comp : components) {
// if (comp instanceof JComponent) {
// comps.add((JComponent) comp);
// JComponent jc = (JComponent) comp;
// final Component[] components2 = ;
// comps.addAll(toList(components2));
// comps.addAll(toList(((JComponent) comp).getComponents()));
// }
// }
//		if (comp instanceof JCheckBox) {
//			cont = new Beschriftung();
//			// cont = buildFrom(((AbstractButton) comp).getAction());
//			((Beschriftung) cont).setShortDescription(((JComponent) comp).getToolTipText());
//		} else if (comp instanceof JLabel) {
//			cont = new Beschriftung();
//			((Beschriftung) cont).setShortDescription(((JComponent) comp).getToolTipText());
//		} else if (comp instanceof AbstractButton) {
//			cont = buildFrom(((AbstractButton) comp).getAction());
//		} else