package eu.gronos.kostenrechner.view;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagLayout;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.FocusAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.Locale;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFormattedTextField;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRootPane;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.border.EmptyBorder;
import javax.swing.event.HyperlinkListener;
import javax.swing.text.NumberFormatter;

import eu.gronos.kostenrechner.controller.BeiFocusMarkierenHorcher;
import eu.gronos.kostenrechner.controller.BeschriebeneAktion;
import eu.gronos.kostenrechner.controller.DialogFensterSchliessenHorcher;
import eu.gronos.kostenrechner.controller.HinzufuegenAbbrechenAction;
import eu.gronos.kostenrechner.controller.HinzufuegenOkAction;
import eu.gronos.kostenrechner.controller.NumberExtractor;
import eu.gronos.kostenrechner.controller.SprungmarkenHorcher;
import eu.gronos.kostenrechner.model.Beschriftung;
import eu.gronos.kostenrechner.model.LangBeschriftung;
import eu.gronos.kostenrechner.model.NameContainerSammlung;
import eu.gronos.kostenrechner.view.result.FontHelferForWindow;

/**
 * Oberklasse für alle Hinzufügen-Dialoge
 * 
 * @author Peter Schuster (setrok)
 * @date: 29.06.2014
 * 
 */
public abstract class HinzufuegenDialog<E> extends JDialog {

	private static final long serialVersionUID = -6595384338088840389L;
	public static final String FEHLER_BT_UNTERLIEGEN_GT_STREITWERT = "Der Beteiligte kann nicht zu mehr verurteilt werden, als von ihm eingeklagt wurde!";
	public static final String FEHLER_WK_STREITWERT = "Bei einer Widerklage muss der Streitwert > 0 sein.";
	public static final String FEHLER_BK_STREITWERT = "Der Streitwert muss über 0 sein.";
	public static final String FEHLER_ZAHLFORMAT_DWBK = "Ungültiges Zahlformat bei der Anzahl der Drittwiderbeklagten";
	public static final String FEHLER_NIX_WIDERKLAEGER_WIDERBEKLAGTER = "Kein widerklagender Beklagter oder kein Widerbeklagter asugewählt!";
	public static final String FEHLER_UNTERLIEGEN_GT_STREITWERT = "Das Unterliegen darf nicht höher als der Streitwert sein! Bei Gesamtschuldnern wird das gesamtschuldnerische Unterliegen mit berücksichtigt.";
	public static final String FEHLER_ZAHLENFORMAT = "Fehler beim Konvertieren des Zahlenformats!";
	public static final String FEHLER_BK_GESAMTSCHULDNERSCHAFT = "Wenn Sie gesamtschuldnerische Verurteilung aktivieren, muss mehr als ein Beklagter markiert sein, die Höhe der Verurteilung > 0 sein, aber maximal so hoch wie der Streitwert sein.";
	public static final String FEHLER_WK_GESAMTSCHULDNERSCHAFT = "Wenn Sie gesamtschuldnerische Verurteilung aktivieren, muss mehr als ein Widerbeklagter markiert sein, die Höhe der Verurteilung > 0 sein, aber maximal so hoch wie der Streitwert sein.";
	public static final String FEHLER_ZAHLENFORMAT_SW = "Falches Zahlformat beim Streitwert!";
	public static final String FEHLER_KLAEGER_NIX_WBK = "Der ausgewählte Kläger ist nicht widerbeklagt.";
	public static final String FEHLER_ZAHLENFORMAT_KL_BK = "Falsches Zahlformat bei Kläger- oder Beklagtenanzahl! ";
	public static final String FEHLER_BEIM_ANLEGEN_KL_BK = "Fehler beim Anlegen der Kläger oder Beklagten! ";
	public static final String FEHLER_AUSLAGENHOEHE_NEGATIV = "Die Auslagenhöhe darf nicht negativ sein!";
	public static final String FEHLER_ANZAHL_NEGATIV = "Die Anzahl darf nicht negativ sein!";
	public static final String FEHLER_ZAHLENFORMAT_WERT = "Falsches Eingabeformat beim Wert! ";
	public static final String FEHLER_BEGRUENDETHEIT_GT_WERT = "Eine Forderung kann nicht erfolgreicher sein als ihr Wert (ne ultra petita)!";
	// " Die Begründetheit der Forderung kann nicht höher als der Wert sein!";

	protected static final EmptyBorder BORDER = new EmptyBorder(5, 5, 5, 5);
	/**
	 * Der JPanel mit dem Namen contentPanel ist das JPanel für den eigentlichen
	 * Inhalt
	 */
	protected final JPanel contentPanel = new JPanel();
	/**
	 * Der boolean mit dem Namen isOk speichert, ob der JDialog mit OK oder Cancel
	 * verlassen wurde.
	 */
	private boolean ok = false;
	private JPanel buttonPane;

	protected NumberFormatter formatter;

	private final AbstractAction okAction = new HinzufuegenOkAction<E>(this,
			(LangBeschriftung) NameContainerSammlung.BESCHRIFTUNGEN.get(76051));
	private final AbstractAction cancelAction = new HinzufuegenAbbrechenAction<E>(this,
			(LangBeschriftung) NameContainerSammlung.BESCHRIFTUNGEN.get(76061));

	private final WindowAdapter fensterSchliesser = new DialogFensterSchliessenHorcher<E>(this);

	public static final FocusAdapter BEI_FOCUS_MARKIEREN = new BeiFocusMarkierenHorcher();
	private JButton okButton;
	private boolean okButtonToBeFocussed = false;
	public static final HyperlinkListener HYPERLINK_LISTENER = new SprungmarkenHorcher();
	private final FontHelferForWindow fufhelfer = new FontHelferForWindow(this);

	/**
	 * Erstellt einen modalen JDialog mit einer buttonPane (OK und Abbrechen) und
	 * einem contentPanel. Dieser muss von der abgeleiteten Klasse in der Methode
	 * {@link #fuelleContentPane()} mit JComponents gefüllt werden, die von
	 * {@link #showDialog()} aufgerufen wird.
	 * 
	 * @param owner the JFrame from which the dialog is displayed
	 * @param title Der anzuzeigende Titel
	 * @throws HeadlessException wenn in einer Umgebung aufgerufen, die keinen
	 *                           Bildschirm, Tastatur oder Maus unterstützt
	 * @Deprecated besser {@link #HinzufuegenDialog(JFrame, Beschriftung)} nutzen
	 */
	@Deprecated
	public HinzufuegenDialog(JFrame owner, String title) throws HeadlessException {
		super(owner, title, true);
		baueContentPane();
		setButtonPane();
		formatter = createNumberFormatter();
	}

	/**
	 * Erstellt einen modalen {@link JDialog} mit einer buttonPane (OK und
	 * Abbrechen) und einem contentPanel. Dieser muss von der abgeleiteten Klasse in
	 * der Methode {@link #fuelleContentPane()} mit {@link JComponent}s gefüllt
	 * werden, die von {@link #showDialog()} aufgerufen wird.
	 * 
	 * @param owner        the JFrame from which the dialog is displayed
	 * @param beschriftung eine {@link Beschriftung}, mit dem der anzuzeigende Titel
	 *                     {@link JDialog#getTitle()} gesetzt wird
	 * @throws HeadlessException wenn in einer Umgebung aufgerufen, die keinen
	 *                           Bildschirm, Tastatur oder Maus unterstützt
	 */
	public HinzufuegenDialog(JFrame owner, Beschriftung beschriftung) throws HeadlessException {
		// super(owner, title, true);beschriftung.getTitle(),
		super(owner, true);
		beschriftung.applyTo(this);
		baueContentPane();
		setButtonPane();
		beschriftung.applyTo(getContentPane());
		formatter = createNumberFormatter();
	}

	/**
	 * Die Methode wird von {@link #okAction} aufgerufen, um zu prüfen, ob alle
	 * JComponents zur Eingabe mit gültigen Werten gefüllt sind und diese zueinander
	 * nicht im Widerspruch stehen.
	 * 
	 * @return true, wenn die Plausibilitäten zufrieden sind
	 */
	public abstract boolean parseEingabe();

	/**
	 * Die Methode showDialog dient dazu, den Dialog aufzurufen, den
	 * {@link #contentPanel} über die Methode {@link #fuelleContentPane()} der
	 * abgeleiteten Klasse mit Komponenten zu füllen, mit {@link #setBounds()} die
	 * Größe des Dialogs anzupassen und auf eine Eingabe zu warten und dann einen E
	 * zurückzugeben. Zur Zurückgabe der Eingabe wird {@link #baueRueckgabewert()}
	 * der abgeleiteten Klasse aufgerufen.
	 * 
	 * @return einen <code>&lt;E&gt;</code> nach den eingegebenen Werten
	 */
	public E showDialog() {
		fuelleContentPane();
		// setBounds();
		pack();
		setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
		addWindowListener(fensterSchliesser);
		// OK-Schaltfläche fokussieren, wenn gewünscht
		if (okButtonToBeFocussed)
			getOkButton().requestFocus();
		setVisible(true);
		// Hier wartet der wohl...
		if (isOk()) {
			return baueRueckgabewert();
		} else {
			return null;
		}
	}

	/**
	 * Die Methode lässt die Position vom Betriebssystem zuweisen und setzt die
	 * Größe auf die bevorzugte Größe. Wird von {@link #showDialog()} aufgerufen.
	 * 
	 * @see java.awt.Window#setBounds(int,int,int,int)
	 * @see java.awt.Window#setLocationByPlatform(boolean)
	 * @see java.awt.Window#setSize(Dimension)
	 * @see java.awt.Window#pack()
	 */
	@Override
	public void pack() {
		setLocationByPlatform(true);
		setSize(fufhelfer.calculatePreferredSizeWith(BORDER.getBorderInsets()));//
		super.pack();
	}

	/**
	 * Die Methode ruft den OK-Button auf
	 * 
	 * @param ae ein ActionEvent
	 */
	public void performOkAction(ActionEvent ae) {
		okAction.actionPerformed(ae);
	}

	/**
	 * Die Methode schließt den Dialog; kann in einer {@link Action} benutzt werden
	 * 
	 */
	public void fensterSchliessenBeiAbbruch() {

		//System.out.println(new AllesBeschrifter().beschreibeComponents(this));

		setOk(false);
		setVisible(false);
		dispose();
	}

	/**
	 * @return gibt {@link #ok} als {@link boolean} zurück.
	 */
	public boolean isOk() {
		return ok;
	}

	/**
	 * @param ok d. {@link #ok}, d. gesetzt werden soll als {@link boolean}.
	 */
	public void setOk(boolean ok) {
		this.ok = ok;
	}

	/**
	 * @return gibt den buttonPane als JPanel zurück.
	 */
	public JPanel getButtonPane() {
		return buttonPane;
	}

	/**
	 * Die Methode dient dazu, aus den Eingabefeldern des Dialogs einen Rückgabewert
	 * zusammenzubauen, der dann von {@link #showDialog()} an die aufrufende Maske
	 * zurückgegeben werden kann.
	 * 
	 * @return ein <code>&lt;E&gt;</code> mit allen Werten
	 */
	protected abstract E baueRueckgabewert();

	/**
	 * Die Methode füllt den Inhaltsbereich des Dialogs mit JComponents zur Eingabe.
	 * Wird von {@link #showDialog()} aufgerufen.
	 * 
	 */
	protected abstract void fuelleContentPane();

	/**
	 * Die Methode initialisiert die allgemeinen JComponents für den Inhaltsbereich
	 * des Dialogs und wird vom Konstruktor selbsttätig aufgerufen.
	 * 
	 */
	protected void baueContentPane() {
		GridBagLayout gbl_contentPanel = new GridBagLayout();
		contentPanel.setBorder(BORDER);
		contentPanel.setLayout(gbl_contentPanel);
		getContentPane().add(contentPanel, BorderLayout.CENTER);
	}

	/**
	 * Die Methode zieht den Zahlwert aus dem übergebenen
	 * {@link JFormattedTextField}
	 * 
	 * @param formattedTextField das {@link JFormattedTextField}, das ausgelesen
	 *                           werden soll.
	 * @param beschriftung       der Text des dazugehörigen {@link JLabel}s
	 * 
	 * @return den ausgelesenen Wert als double
	 */
	protected double doubleAusFormatter(JFormattedTextField formattedTextField, String beschriftung)
			throws NumberFormatException {
		return new NumberExtractor(formattedTextField, beschriftung).extractDouble();
	}

	/**
	 * Die Methode zieht den Zahlwert aus dem übergebenen
	 * {@link JFormattedTextField}
	 * 
	 * @param formattedTextField das {@link JFormattedTextField}, das ausgelesen
	 *                           werden soll.
	 * @param beschriftung       der Text des dazugehörigen {@link JLabel}s
	 * 
	 * @return den ausgelesenen Wert als double
	 */
	protected int intAusFormatter(JFormattedTextField formattedTextField, String beschriftung)
			throws NumberFormatException {
		return new NumberExtractor(formattedTextField, beschriftung).extractInteger();
	}

	/**
	 * Die Methode fügt eine Action der ActionMap und der InputMap der JRootPane des
	 * HinzufuegenDialogs hinzu.
	 * 
	 * @param action eine Action, bei der die Values
	 *               {@link Action#ACTION_COMMAND_KEY} und
	 *               {@link Action#ACCELERATOR_KEY} gesetzt sein müssen.
	 */
	protected void addToRootInputMap(Action action) {
		JRootPane rp = getRootPane();
		/* Zunächst die Action der ActionMap hinzufügen unter ihrem Namen */
		rp.getActionMap().put(action.getValue(Action.ACTION_COMMAND_KEY), action);
		/* Dann die Action der InputMap unter ihrer Tastenkombination hinzufügen */
		rp.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put((KeyStroke) action.getValue(Action.ACCELERATOR_KEY),
				action.getValue(Action.ACTION_COMMAND_KEY));
		// rp.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(
		// KeyStroke.getKeyStroke(
		// (Integer) action.getValue(Action.ACCELERATOR_KEY),
		// InputEvent.ALT_DOWN_MASK),
		// action.getValue(Action.ACTION_COMMAND_KEY));
	}

	/**
	 * Die Methode entfernt eine Action aus der ActionMap und der InputMap der
	 * JRootPane des HinzufuegenDialogs.
	 * 
	 * @param action eine Action, bei der die Values
	 *               {@link Action#ACTION_COMMAND_KEY} und
	 *               {@link Action#ACCELERATOR_KEY} gesetzt sein müssen.
	 */
	protected void removeFromInputMap(Action action) {
		JRootPane rp = getRootPane();
		/* Zuerst die Tastenkombination entfernen */
		rp.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).remove((KeyStroke) action.getValue(Action.ACCELERATOR_KEY));
		/* Dann die Action entfernen */
		rp.getActionMap().remove(action.getValue(Action.ACTION_COMMAND_KEY));
	}

	/**
	 * Ordnet ein JLabel einer JComponent zu und setzt den Mnemonic key.
	 * 
	 * @param label     ein JLabel
	 * @param component the Component this label is for, or null if the label is not
	 *                  the label for a component
	 * @param key       a keycode that indicates a mnemonic key
	 * 
	 * @see javax.swing.JLabel#setLabelFor(Component)
	 * @see javax.swing.JLabel#setDisplayedMnemonic(int)
	 */
	protected void setMnemonicLabelFor(JLabel label, JComponent component, int key) {
		label.setLabelFor(component);
		label.setDisplayedMnemonic(key);
	}

	/**
	 * @return {@link #okButtonToBeFocussed}, also true, wenn bei Aufruf der Maske
	 *         der OK-Button den Focus haben soll, sonst false.
	 */
	protected boolean isOkButtonToBeFocussed() {
		return okButtonToBeFocussed;
	}

	/**
	 * Die Methode soll die OK-Schaltfläche fokussieren, wenn gewünscht
	 * 
	 * @param okButtonToBeFocussed true, wenn bei Aufruf der Maske der OK-Button den
	 *                             Focus haben soll
	 */
	protected void setOkButtonToBeFocussed(boolean okButtonToBeFocussed) {
		this.okButtonToBeFocussed = okButtonToBeFocussed;
	}

	/**
	 * @return gibt die okAction als AbstractAction zurück.
	 */
	protected AbstractAction getOkAction() {
		return okAction;
	}

	/**
	 * Die Methode aktiviert oder deaktiviert den Ok-Button
	 * 
	 * @param enabled true oder false
	 */
	protected void setOkActionEnabled(boolean enabled) {
		okAction.setEnabled(enabled);
	}

	/**
	 * Setzt für ein {@link JFormattedTextField} die {@link AbstractAction} in die
	 * ActionMap und InputMap, die bei Enter ausgeführt werden soll. Zudem fügt die
	 * Methode das Verhalten hinzu, dass bei FocusErlangung der Inhalt markiert
	 * wird.
	 * 
	 * @param field  das {@link JTextField}, dessen Entertaste belegt werden soll
	 * @param action die {@link AbstractAction}, die bei Entertaste ausgeführt
	 *               werden soll
	 */
	protected void setEnterAction(JTextField field, AbstractAction action) {
		if (action != null) {
			field.getInputMap(JComponent.WHEN_FOCUSED).put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0),
					action.getValue(Action.ACTION_COMMAND_KEY));
			field.getActionMap().put(action.getValue(Action.ACTION_COMMAND_KEY), action);
		}
		field.setHorizontalAlignment(JTextField.RIGHT);
		field.addFocusListener(BEI_FOCUS_MARKIEREN);
	}

	/**
	 * @return gibt {@link #okButton} als {@link JButton} zurück.
	 */
	protected JButton getOkButton() {
		return okButton;
	}

	/**
	 * Die Methode baut einen Numberformatter. Der kann für Nummereingabefelder
	 * benutzt werden. Grouping ist ausgeschaltet, Eingaben können fehlerhaft sein.
	 * 
	 * @return den {@link NumberFormatter}
	 */
	private NumberFormatter createNumberFormatter() {
		NumberFormat format = DecimalFormat.getInstance(Locale.GERMAN);
		// NumberFormat format = NumberFormat.getInstance();
		NumberFormatter formatter = new NumberFormatter(format);
		format.setGroupingUsed(false);
		formatter.setAllowsInvalid(true);
		// formatter.setAllowsInvalid(false);
		return formatter;
	}

	/**
	 * Die Methode setzt die OK- und die Abbrechen-Schaltfläche in einen eigenen
	 * JPanel des Dialogs
	 * 
	 */
	private void setButtonPane() {
		buttonPane = new JPanel();
		buttonPane.setLayout(new FlowLayout(FlowLayout.RIGHT));
		getContentPane().add(buttonPane, BorderLayout.SOUTH);
		setOkButton(((BeschriebeneAktion) okAction).toButton());
		// new JButton(okAction));
		addToRootInputMap(okAction);
		buttonPane.add(getOkButton());
		getRootPane().setDefaultButton(getOkButton());
		JButton cancelButton = ((BeschriebeneAktion) cancelAction).toButton();
		// new JButton(cancelAction);
		buttonPane.add(cancelButton);
		addToRootInputMap(cancelAction);
	}

	/**
	 * @param okButton d. {@link #okButton}, d. gesetzt werden soll als
	 *                 {@link JButton}.
	 */
	private void setOkButton(JButton okButton) {
		this.okButton = okButton;
	}

	/**
	 * @return gibt {@link #cancelAction} als {@link AbstractAction} zurück.
	 */
	protected AbstractAction getCancelAction() {
		return cancelAction;
	}
}
// , KeyEvent.VK_A,KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0)
//"OK","OK", 
//"Bestätigt die Eingabe (Eingabetaste).", 
//	, "Abbrechen","Cancel", 
//"Bricht die Eingabe ab (ESC).", 